@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-from astropy.samp import SAMPIntegratedClient
+from astropy.samp import SAMPIntegratedClient, conf as samp_conf
 from astropy.samp.errors import SAMPHubError, SAMPProxyError
 from datetime import datetime, UTC
 from pathlib import Path
@@ -32,8 +32,11 @@ class DS9:
                  poll_alive_time=5,                             # watcher thread poll time (seconds)
                  init_retry_time=1,                             # time to sleep between retries on init (seconds)
                  debug=False,                                   # debug output (very verbose)
-                 samp_hub_file=None                             # use samp_hub_file from an existing hub, rather than start a dedicated one attached to ds9
-                ):
+                 singleton=True,                                # if true, creates a truly unique instance, using time, PID. if false, creates an instance based on name and DISPLAY, that can be reattached
+                 samp_hub_file=None,                            # use samp_hub_file from an existing hub, rather than start a dedicated one attached to ds9
+                 samp_conf_use_internet=False,                  # Whether to allow `astropy.samp` to use the internet, if available (conf default: True)
+                 samp_conf_n_retries=None,                      # How many times to retry communications when they fail (conf default: 10)
+                 ):
         self.debug = debug
         self.exit_callback = exit_callback
         self.kill_ds9_on_exit = kill_ds9_on_exit
@@ -56,8 +59,13 @@ class DS9:
                 samp_hub_cmd = '-samp hub yes'
                 # generate a unique SAMP_HUB from title, timestamp, process PID
                 Path(SAMP_HUB_PATH).mkdir(mode=0o700, parents=True, exist_ok=True)
-                tnow = datetime.now(UTC)
-                samp_hub_name = f"ds9_{title}_utc{tnow.strftime('%Y%m%dT%H%M%S')}.{tnow.microsecond:06d}_pid{self.__pid}"
+                user = os.environ.get('USER', 'anonymous')
+                display = os.environ.get('DISPLAY', 'headless')
+                if singleton:
+                    tnow = datetime.now(UTC)
+                    samp_hub_name = f"ds9_{title}_{user}_{display}_utc{tnow.strftime('%Y%m%dT%H%M%S')}.{tnow.microsecond:06d}_pid{self.__pid}"
+                else:
+                    samp_hub_name = f"ds9_{title}_{user}_{display}"
                 samp_hub_name = re.sub(r'[^A-Za-z0-9\.:]', '_', samp_hub_name) # sanitized
                 samp_hub_file = self.__samp_hub_file = f"{SAMP_HUB_PATH}/{samp_hub_name}.samp"
                 ds9_ishub = True
@@ -65,7 +73,7 @@ class DS9:
                 hub_timeout = timeout
 
             if noiraf: ds9args += ' -xpa no -unix none -fifo none -port 0'
-
+            if display := os.environ.get('DISPLAY'): ds9args += f" -display '{display}'"
             cmd = f"{DS9_EXE} -samp client yes {samp_hub_cmd} -samp web hub no -title '{title}' {ds9args}"
             os.environ['SAMP_HUB'] = f"std-lockurl:file://{samp_hub_file}"
             os.environ['XMODIFIERS'] = '@im=none' # fix ds9 (Tk) responsiveness on Wayland. see https://github.com/ibus/ibus/issues/2324#issuecomment-996449177
@@ -218,13 +226,26 @@ class DS9:
         self.exit(main_thread=False)
         if self.debug: print('watch_thread exit')
 
-    def set(self, *cmds, timeout=10):
+    # timeout=0 wait for ever
+    # timeout<0 do not wait
+    def set(self, *cmds, timeout=10, batch_timeout=False):
         with self.__lock:
+            timed_out_cmds = []
             for cmd in cmds:
                 if self.debug: print(f"set {cmd}")
-                self.__samp.ecall_and_wait(self.__samp_clientId, 'ds9.set', f"{int(timeout)}", cmd=cmd)
-
-    def get(self, cmd, timeout=0): # some commands like 'iexam key coordinate' are blocking, use timeout=0 by default XXX chnage the default to a reasonable value, use 0 when needed by the user
+                try:
+                    if timeout >= 0:
+                        self.__samp.ecall_and_wait(self.__samp_clientId, 'ds9.set', f"{int(timeout)}", cmd=cmd)
+                    else:
+                        self.__samp.ecall(self.__samp_clientId, 'samp::sync::call', 'ds9.set', cmd=cmd)
+                except Exception as e:
+                    if batch_timeout and type(e) == SAMPProxyError and e.faultString == 'Timeout expired!':
+                        timed_out_cmds.append(cmd)
+                    else:
+                        raise
+            if batch_timeout and timed_out_cmds:
+                raise TimeoutError(f"batch cmds {', '.join(timed_out_cmds)} timed out!") from None
+    def get(self, cmd, timeout=0): # some commands like 'iexam key coordinate' are blocking, use timeout=0 (wait forever) by default
         with self.__lock:
             if self.debug: print(f"get {cmd}")
             rc = self.__samp.ecall_and_wait(self.__samp_clientId, 'ds9.get', f"{int(timeout)}", cmd=cmd)
